using System;
using UnityEngine;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using SimpleJSON;
using System.Linq;

namespace ky1001
{
    public class HandMorphManager : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "ky1001";
        public string pluginName = "HandMorphManager";
        public string pluginVersion = "8.0";
        public string pluginDate = "2022-06-08";
        public string pluginDescription = "";
        #endregion

        #region Vars
        private Atom CurrentAtom;

        private const string _saveExt = "vap";
        private const string _handmorphSavePath = "Saves\\PluginData\\handmorph";
        private const string _vapPosePath = "Custom\\Atom\\Person\\Pose";
        private const string _jsonPosePath = "Saves\\Person\\pose";
        //private string SaveData;
        private string LoadData;
        private string ext;
        private string jsonPath;

        private bool CSH;
        private bool CSF;
        private bool CSS;

        private UIDynamic spacer1;
        private UIDynamic spacer2;
        private UIDynamic spacer3;

        private JSONStorableBool toggle_x;
        private JSONStorableBool toggle_l;
        private JSONStorableBool toggle_r;
        private JSONStorableString filename;

        private JSONStorableBool toggle_es;
        private JSONStorableBool toggle_el;
        private JSONStorableBool toggle_er;

        private JSONNode jSONNode;
        private JSONArray atomArray;
        private JSONArray asArray;
        private JSONArray morphsAsArray;
        private JSONClass asAtomObject;
        private JSONClass asObject;
        private JSONClass morphs;
        private string id;

        private readonly string[] HandMorphs = new string[] {
            "Hand Fist",
            "Hand Grasp",
            "Hand Spread",
            "Hand Chop",
            "Hand Straighten",
            "Fingers Fist",
            "Fingers Grasp",
            "Fingers In-Out",
            "Fingers Squeeze",
            "Fingers Straighten",
            "Thumb Bend",
            "Index Finger Bend",
            "Mid Finger Bend",
            "Ring Finger Bend",
            "Pinky Finger Bend",
            "Thumb Fist",
            "Thumb Grasp",
            "Thumb In-Out",
            "Thumb Straighten",
        };
        private List<string> HandMorphList;
        #endregion

        #region Controls
        private UIDynamicButton button;
        private UIDynamicButton Getbutton;
        private UIDynamicButton clbutton;
        private UIDynamicButton crbutton;
        private UIDynamicButton swbutton;
        private UIDynamicButton loadbutton;
        private UIDynamicButton savebutton;
        private UIDynamicButton loadVapButton;
        private UIDynamicButton loadJsonButton;
        private UIDynamicButton cshbutton;
        private UIDynamicButton csfbutton;
        private UIDynamicButton cssbutton;

        private UIDynamicToggle toggleel;
        private UIDynamicToggle toggleer;

        private JSONStorableFloat[] lMorphName;
        private JSONStorableFloat[] rMorphName;
        private UIDynamicSlider[] lSlider;
        private UIDynamicSlider[] rSlider;
        private float[] lSliderPast;
        private float[] rSliderPast;

        private JSONStorable geometry;
        private DAZCharacterSelector character;
        private GenerateDAZMorphsControlUI morphControl;
        #endregion

        public override void Init()
        {
            try
            {
                #region Button
                var loadPresetUI = CreateButton("Load Preset");
                loadPresetUI.button.onClick.AddListener(() =>
                {
                    filename.val = "";
                    FileManagerSecure.CreateDirectory(_handmorphSavePath);
                    var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_handmorphSavePath, false, false, true, true);
                    SuperController.singleton.GetMediaPathDialog(HandleLoadPreset, "vap|json|handmorph", _handmorphSavePath, true, true, false, null, false, shortcuts);
                });
                JSONStorableAction LoadDatas = new JSONStorableAction("Load Hand Morphs without dialog", () =>
                {
                    HandleLoadPreset("");
                });
                RegisterAction(LoadDatas);

                var savePresetUI = CreateButton("Save Preset", true);
                savePresetUI.button.onClick.AddListener(() =>
                {
                    FileManagerSecure.CreateDirectory(_handmorphSavePath);
                    var fileBrowserUI = SuperController.singleton.fileBrowserUI;
                    fileBrowserUI.SetTitle("Save Hand Morphs preset");
                    fileBrowserUI.fileRemovePrefix = null;
                    fileBrowserUI.hideExtension = false;
                    fileBrowserUI.keepOpen = false;
                    fileBrowserUI.fileFormat = "vap|json";
                    fileBrowserUI.defaultPath = _handmorphSavePath;
                    fileBrowserUI.showDirs = true;
                    fileBrowserUI.shortCuts = null;
                    fileBrowserUI.browseVarFilesAsDirectories = true;
                    fileBrowserUI.SetTextEntry(true);
                    fileBrowserUI.Show(HandleSavePreset);
                    fileBrowserUI.fileEntryField.text = (int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds + "." + _saveExt;
                    fileBrowserUI.ActivateFileNameField();
                    fileBrowserUI.showInstallFolderInDirectoryList = true;
                });

                var loadVapPose = CreateButton("Load Vap Pose Preset");
                loadVapPose.button.onClick.AddListener(() =>
                {
                    filename.val = "";
                    var vapPoseShortcuts = FileManagerSecure.GetShortCutsForDirectory(_vapPosePath, false, false, true, true);
                    SuperController.singleton.GetMediaPathDialog(HandleLoadPreset, "vap", _vapPosePath, true, true, false, null, false, vapPoseShortcuts);
                });

                var loadJsonPose = CreateButton("Load Json Pose Preset", true);
                loadJsonPose.button.onClick.AddListener(() =>
                {
                    filename.val = "";
                    var jsonPoseShortcuts = FileManagerSecure.GetShortCutsForDirectory(_jsonPosePath, false, false, true, true);
                    SuperController.singleton.GetMediaPathDialog(HandleLoadPreset, "json", _jsonPosePath, true, true, false, null, false, jsonPoseShortcuts);
                });

                button = CreateButton("Hand Morphs to Default");
                if (button != null)
                {
                    button.button.onClick.AddListener(DefaultHandMorphs);
                }
                JSONStorableAction DefaultMorphs = new JSONStorableAction("Hand Morphs Set to Default", () =>
                {
                    DefaultHandMorphs();
                });
                RegisterAction(DefaultMorphs);

                Getbutton = CreateButton("Hand Morphs to Zero", true);
                if (Getbutton != null)
                {
                    Getbutton.button.onClick.AddListener(ZeroHandMorps);
                }
                JSONStorableAction ZeroMorphs = new JSONStorableAction("Hand Morphs Set to Zero", () =>
                {
                    ZeroHandMorps();
                });
                RegisterAction(ZeroMorphs);

                clbutton = CreateButton("Copy Left to Right", false);
                if (clbutton != null)
                {
                    clbutton.button.onClick.AddListener(CopyLtoR);
                }
                JSONStorableAction CopyLR = new JSONStorableAction("Hand Morphs Copy Left to Right", () =>
                {
                    CopyLtoR();
                });
                RegisterAction(CopyLR);

                crbutton = CreateButton("Copy Right to Left", true);
                if (crbutton != null)
                {
                    crbutton.button.onClick.AddListener(CopyRtoL);
                }
                JSONStorableAction CopyRL = new JSONStorableAction("Hand Morphs Copy Right to Left", () =>
                {
                    CopyRtoL();
                });
                RegisterAction(CopyRL);

                swbutton = CreateButton("Swap Left and Right", false);
                if (swbutton != null)
                {
                    swbutton.button.onClick.AddListener(SwapLandR);
                }
                JSONStorableAction SwapLR = new JSONStorableAction("Hand Morphs Swap Left and Right", () =>
                {
                    SwapLandR();
                });
                RegisterAction(SwapLR);

                toggle_x = new JSONStorableBool("Enable swap loading Preset", false);
                CreateToggle(toggle_x, true);
                RegisterBool(toggle_x);

                toggle_l = new JSONStorableBool("Enable loading Preset to Left", true);
                CreateToggle(toggle_l, false);
                RegisterBool(toggle_l);
                toggle_r = new JSONStorableBool("Enable loading Preset to Right", true);
                CreateToggle(toggle_r, true);
                RegisterBool(toggle_r);

                toggle_es = new JSONStorableBool("Enable Saveing L-R Separately", false);
                toggle_el = new JSONStorableBool("Enable Saveing Preset from Left", true);
                toggle_er = new JSONStorableBool("Enable Saveing Preset from Right", true);

                if (toggle_es.val)
                {
                    CreateToggle(toggle_el, false);
                    CreateToggle(toggle_er, true);
                }

                filename = new JSONStorableString("Hand Morph Manager Filename", "Saves\\PluginData\\handmorph\\");
                RegisterString(filename);
                #endregion

                geometry = containingAtom.GetStorableByID("geometry");
                character = geometry as DAZCharacterSelector;
                morphControl = character.morphsControlUI;
                HandMorphList = new List<string>(HandMorphs);

                lMorphName = new JSONStorableFloat[19];
                rMorphName = new JSONStorableFloat[19];
                lSlider = new UIDynamicSlider[19];
                rSlider = new UIDynamicSlider[19];

                lSliderPast = new float[19];
                rSliderPast = new float[19];

                CSH = true;
                CreateButtonCSH();
                CreateSlidersCSH();

                CSF = true;
                CreateButtonCSF();
                CreateSlidersCSF();

                CSS = true;
                CreateButtonCSS();
                CreateSlidersCSS();

                for (var i = 0; i < HandMorphs.Count(); i++)
                {
                    RegisterFloat(lMorphName[i]);
                    RegisterFloat(rMorphName[i]);
                }

                morphValueToSlider();
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        private void DefaultHandMorphs()
        {
            morphControl.GetMorphByDisplayName("Right Hand Fist").morphValue = 0f;
            morphControl.GetMorphByDisplayName("Left Hand Fist").morphValue = 0f;
            Invoke("DefaultHandMorphsMain", 0.2f);
        }

        private void DefaultHandMorphsMain()
        {
            foreach (string HandMorphs in HandMorphList)
            {
                if (HandMorphs != "Hand Fist")
                {
                    morphControl.GetMorphByDisplayName("Right " + HandMorphs).SetDefaultValue();
                    morphControl.GetMorphByDisplayName("Left " + HandMorphs).SetDefaultValue();
                }
            }
            morphValueToSlider();
        }

        private void ZeroHandMorps()
        {
            foreach (string HandMorphs in HandMorphList)
            {
                morphControl.GetMorphByDisplayName("Right " + HandMorphs).morphValue = 0f;
                morphControl.GetMorphByDisplayName("Left " + HandMorphs).morphValue = 0f;
            }
            morphValueToSlider();
        }

        private void morphValueToSlider()
        {
            for (int i = 0; i < 19; i++)
            {
                lSlider[i].slider.value = morphControl.GetMorphByDisplayName("Left " + HandMorphs[i]).morphValue;
                rSlider[i].slider.value = morphControl.GetMorphByDisplayName("Right " + HandMorphs[i]).morphValue;
                lSliderPast[i] = lSlider[i].slider.value;
                rSliderPast[i] = rSlider[i].slider.value;
            }
        }

        private void sliderValueToMorph()
        {
            for (int i = 0; i < 19; i++)
            {
                if (lSliderPast[i] != lSlider[i].slider.value)
                {
                    morphControl.GetMorphByDisplayName("Left " + HandMorphs[i]).morphValue = lSlider[i].slider.value;
                    lSliderPast[i] = lSlider[i].slider.value;
                }
                else
                {
                    lSlider[i].slider.value = morphControl.GetMorphByDisplayName("Left " + HandMorphs[i]).morphValue;
                }

                if (rSliderPast[i] != rSlider[i].slider.value)
                {
                    morphControl.GetMorphByDisplayName("Right " + HandMorphs[i]).morphValue = rSlider[i].slider.value;
                    rSliderPast[i] = rSlider[i].slider.value;
                }
                else
                {
                    rSlider[i].slider.value = morphControl.GetMorphByDisplayName("Right " + HandMorphs[i]).morphValue;
                }
            }
        }

        private void CopyLtoR()
        {
            foreach (string HandMorphs in HandMorphList)
            {
                morphControl.GetMorphByDisplayName("Right " + HandMorphs).morphValue = morphControl.GetMorphByDisplayName("Left " + HandMorphs).morphValue;
            }
            morphValueToSlider();
        }

        private void CopyRtoL()
        {
            foreach (string HandMorphs in HandMorphList)
            {
                morphControl.GetMorphByDisplayName("Left " + HandMorphs).morphValue = morphControl.GetMorphByDisplayName("Right " + HandMorphs).morphValue;
            }
            morphValueToSlider();
        }

        private void SwapLandR()
        {
            foreach (string HandMorphs in HandMorphList)
            {
                float l_temp = morphControl.GetMorphByDisplayName("Left " + HandMorphs).morphValue;
                morphControl.GetMorphByDisplayName("Left " + HandMorphs).morphValue = morphControl.GetMorphByDisplayName("Right " + HandMorphs).morphValue;
                morphControl.GetMorphByDisplayName("Right " + HandMorphs).morphValue = l_temp;
            }
            morphValueToSlider();
        }

        private void CreateSlidersCSH()
        {
            for (int i = 0; i < 5; i++)
            {
                if (i == 3)
                {
                    lMorphName[i] = new JSONStorableFloat("Left " + HandMorphs[i], 0, 0f, 1f, true);
                    rMorphName[i] = new JSONStorableFloat("Right " + HandMorphs[i], 0, 0f, 1f, true);
                }
                else
                {
                    lMorphName[i] = new JSONStorableFloat("Left " + HandMorphs[i], 0, -1f, 1f, true);
                    rMorphName[i] = new JSONStorableFloat("Right " + HandMorphs[i], 0, -1f, 1f, true);
                }
                lSlider[i] = CreateSlider(lMorphName[i], false);
                rSlider[i] = CreateSlider(rMorphName[i], true);
            }
        }

        private void CreateSlidersCSF()
        {
            for (int i = 5; i < 10; i++)
            {
                if (i == 8)
                {
                    lMorphName[i] = new JSONStorableFloat("Left " + HandMorphs[i], 0, 0f, 1f, true);
                    rMorphName[i] = new JSONStorableFloat("Right " + HandMorphs[i], 0, 0f, 1f, true);
                }
                else
                {
                    lMorphName[i] = new JSONStorableFloat("Left " + HandMorphs[i], 0, -1f, 1f, true);
                    rMorphName[i] = new JSONStorableFloat("Right " + HandMorphs[i], 0, -1f, 1f, true);
                }
                lSlider[i] = CreateSlider(lMorphName[i], false);
                rSlider[i] = CreateSlider(rMorphName[i], true);
            }
        }

        private void CreateSlidersCSS()
        {
            for (int i = 10; i < 19; i++)
            {
                lMorphName[i] = new JSONStorableFloat("Left " + HandMorphs[i], 0, -1f, 1f, true);
                rMorphName[i] = new JSONStorableFloat("Right " + HandMorphs[i], 0, -1f, 1f, true);
                lSlider[i] = CreateSlider(lMorphName[i], false);
                rSlider[i] = CreateSlider(rMorphName[i], true);
            }
        }

        private void RemoveButtonES()
        {
            RemoveToggle(toggle_el);
            RemoveToggle(toggle_er);
        }

        private void CreateButtonES()
        {
            if (toggle_es.val == true)
            {
                toggleel = CreateToggle(toggle_el, false);
                toggle_el.toggle.onValueChanged.AddListener(delegate { toggle_el_ChangeColor(); });
                if (toggle_el.val == true)
                {
                    toggle_el_e();
                }
                else
                {
                    toggle_el_d();
                }
                toggleer = CreateToggle(toggle_er, true);
                toggle_er.toggle.onValueChanged.AddListener(delegate { toggle_er_ChangeColor(); });
                if (toggle_er.val == true)
                {
                    toggle_er_e();
                }
                else
                {
                    toggle_er_d();
                }
            }
        }

        private void toggle_el_e()
        {
            toggleel.backgroundColor = Color.white;
            toggleel.label = "Enable Saveing Preset from Left";
            toggleel.textColor = Color.black;
        }

        private void toggle_el_d()
        {
            toggleel.backgroundColor = Color.red;
            toggleel.label = "Disable Saveing Preset from Left";
            toggleel.textColor = Color.white;
        }

        private void toggle_er_e()
        {
            toggleer.backgroundColor = Color.white;
            toggleer.label = "Enable Saveing Preset from Right";
            toggleer.textColor = Color.black;
        }

        private void toggle_er_d()
        {
            toggleer.backgroundColor = Color.red;
            toggleer.label = "Disable Saveing Preset from Right";
            toggleer.textColor = Color.white;
        }

        private void toggle_el_ChangeColor()
        {
            if (toggle_el.val == true)
            {
                toggle_el_e();
            }
            else
            {
                toggle_el_d();
            }
        }

        private void toggle_er_ChangeColor()
        {
            if (toggle_er.val == true)
            {
                toggle_er_e();
            }
            else
            {
                toggle_er_d();
            }
        }

        private void OpenES()
        {
            RemoveButtonES();
            CreateButtonES();
            RemoveButtonCSH();
            RemoveButtonCSF();
            RemoveButtonCSS();
            RemoveSlidersCSH();
            RemoveSlidersCSF();
            RemoveSlidersCSS();

            CreateButtonCSH();
            CreateButtonCSF();
            CreateButtonCSS();

            CSH = !CSH;
            OpenCSH();
            CSF = !CSF;
            OpenCSF();
            CSS = !CSS;
            OpenCSS();
        }

        private void OpenCSH()
        {
            if (CSH == true)
            {
                CSH = false;
                cshbutton.label = "Open Hand Morph Sliders";
                cshbutton.buttonColor = Color.green;
                RemoveSlidersCSH();
            }
            else
            {
                RemoveSlidersCSH();
                RemoveSlidersCSF();
                RemoveSlidersCSS();
                CSH = true;
                CreateButtonCSH();
                CreateSlidersCSH();
                CreateButtonCSF();
                if (CSF == true)
                {
                    CreateSlidersCSF();
                }
                CreateButtonCSS();
                if (CSS == true)
                {
                    CreateSlidersCSS();
                }
            }
            morphValueToSlider();
        }

        private void OpenCSF()
        {
            if (CSF == true)
            {
                CSF = false;
                csfbutton.label = "Open Fingers Morph Sliders";
                csfbutton.buttonColor = Color.green;
                RemoveSlidersCSF();
            }
            else
            {
                RemoveSlidersCSH();
                RemoveSlidersCSF();
                RemoveSlidersCSS();
                CSF = true;
                CreateButtonCSH();
                if (CSH == true)
                {
                    CreateSlidersCSH();
                }
                CreateButtonCSF();
                CreateSlidersCSF();
                CreateButtonCSS();
                if (CSS == true)
                {
                    CreateSlidersCSS();
                }
            }
            morphValueToSlider();
        }

        private void OpenCSS()
        {
            if (CSS == true)
            {
                CSS = false;
                cssbutton.label = "Open Individual Finger Morph Sliders";
                cssbutton.buttonColor = Color.green;
                RemoveSlidersCSS();
            }
            else
            {
                RemoveSlidersCSH();
                RemoveSlidersCSF();
                RemoveSlidersCSS();
                CSS = true;
                CreateButtonCSH();
                if (CSH == true)
                {
                    CreateSlidersCSH();
                }
                CreateButtonCSF();
                if (CSF == true)
                {
                    CreateSlidersCSF();
                }
                CreateButtonCSS();
                CreateSlidersCSS();
            }
            morphValueToSlider();
        }

        private void CreateButtonCSH()
        {
            RemoveButton(cshbutton);
            RemoveToggle(toggle_es);

            cshbutton = CreateButton("Open Hand Morph Sliders", false);
            cshbutton.buttonColor = Color.green;
            if (CSH == true)
            {
                cshbutton.label = "Close Hand Morph Sliders";
                cshbutton.buttonColor = Color.yellow;
            }
            if (cshbutton != null)
            {
                cshbutton.button.onClick.AddListener(OpenCSH);
            }
            CreateToggle(toggle_es, true);
            toggle_es.toggle.onValueChanged.AddListener(delegate { OpenES(); });
        }

        private void CreateButtonCSF()
        {
            RemoveButton(csfbutton);
            RemoveSpacer(spacer2);

            csfbutton = CreateButton("Open Fingers Morph Sliders", false);
            csfbutton.buttonColor = Color.green;
            if (CSF == true)
            {
                csfbutton.label = "Close Fingers Morph Sliders";
                csfbutton.buttonColor = Color.yellow;
            }
            if (csfbutton != null)
            {
                csfbutton.button.onClick.AddListener(OpenCSF);
            }
            spacer2 = CreateSpacer(true);
            spacer2.height = 50f;
        }

        private void CreateButtonCSS()
        {
            RemoveButton(cssbutton);
            RemoveSpacer(spacer3);

            cssbutton = CreateButton("Open Individual Finger Morph Sliders", false);
            cssbutton.buttonColor = Color.green;
            if (CSS == true)
            {
                cssbutton.label = "Close Individual Finger Morph Sliders";
                cssbutton.buttonColor = Color.yellow;
            }
            if (cssbutton != null)
            {
                cssbutton.button.onClick.AddListener(OpenCSS);
            }
            spacer3 = CreateSpacer(true);
            spacer3.height = 50f;
        }

        private void RemoveButtonCSH()
        {
            RemoveButton(cshbutton);
            RemoveSpacer(spacer1);
        }

        private void RemoveButtonCSF()
        {
            RemoveButton(csfbutton);
            RemoveSpacer(spacer2);
        }

        private void RemoveButtonCSS()
        {
            RemoveButton(cssbutton);
            RemoveSpacer(spacer3);
        }

        private void RemoveSlidersCSH()
        {
            for (int i = 0; i < 5; i++)
            {
                RemoveSlider(lMorphName[i]);
                RemoveSlider(rMorphName[i]);
            }
        }

        private void RemoveSlidersCSF()
        {
            for (int i = 5; i < 10; i++)
            {
                RemoveSlider(lMorphName[i]);
                RemoveSlider(rMorphName[i]);
            }
        }

        private void RemoveSlidersCSS()
        {
            for (int i = 10; i < 19; i++)
            {
                RemoveSlider(lMorphName[i]);
                RemoveSlider(rMorphName[i]);
            }
        }

        #region Preset Load and Save
        private void HandleLoadPreset(string path)
        {
            if (filename.val != "")
            {
                filename.val = filename.val.Replace("\\", "/");
                ext = "";
                if (filename.val.Length > 10)
                {
                    if (filename.val.Substring(filename.val.Length - 10, 10) == ".handmorph")
                    {
                        ext = ".handmorph";
                    }
                }

                if (filename.val.Length > 5)
                {
                    if (filename.val.Substring(filename.val.Length - 5, 5) == ".json")
                    {
                        ext = ".json";
                    }
                }

                if (filename.val.Length > 4)
                {
                    if (filename.val.Substring(filename.val.Length - 4, 4) == ".vap")
                    {
                        ext = ".vap";
                    }
                }

                if (ext == "")
                {
                    filename.val = filename.val + ".json";
                }

                if (filename.val.IndexOf(":") == -1)
                {
                    if (filename.val.Substring(0, 6) != "Saves/" && filename.val.Substring(0, 7) != "Custom/")
                    {
                        path = "Saves/PluginData/handmorph/" + filename.val;
                    }
                    else
                    {
                        path = filename.val;
                    }
                }
                else
                {
                    path = filename.val;
                }
            }

            LoadData = SuperController.singleton.ReadFileIntoString(path);
            if (!LoadData.Contains("morphs"))
            {
                if (ext == ".handmorph" | path.Substring(path.Length - 10, 10) == ".handmorph" | (!LoadData.Contains("{")))
                {
                    LoadData = LoadData.Replace(",", "\":\"").Replace("\n", "\",\n\"");
                    LoadData = "{\n\"" + LoadData + "end\n}";
                    LoadData = LoadData.Replace(",\n\"end", "");
                    //SuperController.LogMessage(LoadData);
                }

                jSONNode = JSON.Parse(LoadData);
                foreach (var Item in HandMorphs)
                {
                    if (toggle_x.val)
                    {
                        if (toggle_l.val && jSONNode["Right " + Item] != null)
                        {
                            morphControl.GetMorphByDisplayName("Left " + Item).morphValue = float.Parse(jSONNode["Right " + Item]);
                        }

                        if (toggle_r.val && jSONNode["Left " + Item] != null)
                        {
                            morphControl.GetMorphByDisplayName("Right " + Item).morphValue = float.Parse(jSONNode["Left " + Item]);
                        }
                    }
                    else
                    {
                        if (toggle_l.val && jSONNode["Left " + Item] != null)
                        {

                            morphControl.GetMorphByDisplayName("Left " + Item).morphValue = float.Parse(jSONNode["Left " + Item]);
                        }

                        if (toggle_r.val && jSONNode["Right " + Item] != null)
                        {
                            morphControl.GetMorphByDisplayName("Right " + Item).morphValue = float.Parse(jSONNode["Right " + Item]);
                        }
                    }
                }
            }
            else
            {
                jSONNode = JSON.Parse(LoadData);
                bool jsonExt = (path.Substring(path.Length - 4, 4) != ".vap");
                if (jsonExt)
                {
                    atomArray = jSONNode["atoms"].AsArray;
                    for (int k = 0; k < atomArray.Count; k++)
                    {
                        asAtomObject = atomArray[k].AsObject;
                        id = asAtomObject["id"];
                        string type = asAtomObject["type"];
                        if (type == "Person")
                        {
                            asArray = asAtomObject["storables"].AsArray;
                        }
                    }
                }
                else
                {
                    asArray = jSONNode["storables"].AsArray;
                }

                bool def = Convert.ToBoolean(jSONNode["setUnlistedParamsToDefault"]);
                foreach (var item in HandMorphs)
                {
                    if (toggle_l.val && (def | jsonExt))
                    {
                        morphControl.GetMorphByDisplayName("Left " + item).SetDefaultValue();
                    }

                    if (toggle_r.val && (def | jsonExt))
                    {
                        morphControl.GetMorphByDisplayName("Right " + item).SetDefaultValue();
                    }
                }
                //asArray = jSONNode["storables"].AsArray;
                for (int i = 0; i < asArray.Count; i++)
                {
                    asObject = asArray[i].AsObject;
                    id = asObject["id"];
                    if (id == "geometry")
                    {
                        morphsAsArray = asObject["morphs"].AsArray;
                        for (int j = 0; j < morphsAsArray.Count; j++)
                        {
                            morphs = morphsAsArray[j].AsObject;
                            string morphsName = morphs["name"];
                            string morphsValue = morphs["value"];
                            if (HandMorphs.Contains(morphsName.Replace("Right ", "")) | HandMorphs.Contains(morphsName.Replace("Left ", "")))
                            {
                                if (toggle_x.val)
                                {
                                    if (morphsName.Contains("Right"))
                                    {
                                        if (toggle_l.val)
                                        {
                                            morphControl.GetMorphByDisplayName(morphsName.Replace("Right", "Left")).morphValue = float.Parse(morphsValue);
                                        }
                                    }
                                    else
                                    {
                                        if (toggle_r.val)
                                        {
                                            morphControl.GetMorphByDisplayName(morphsName.Replace("Left", "Right")).morphValue = float.Parse(morphsValue);
                                        }
                                    }
                                }
                                else
                                {
                                    if (toggle_l.val && morphsName.Contains("Left"))
                                    {
                                        morphControl.GetMorphByDisplayName(morphsName).morphValue = float.Parse(morphsValue);
                                    }
                                    if (toggle_r.val && morphsName.Contains("Right"))
                                    {
                                        morphControl.GetMorphByDisplayName(morphsName).morphValue = float.Parse(morphsValue);
                                    }
                                }
                            }
                        }
                    }
                }

            }
            morphValueToSlider();
        }

        private void HandleSavePreset(string path)
        {
            JSONClass morphFile = new JSONClass();

            if (path.Substring(path.Length - 5, 5) == ".json")
            {
                foreach (string HandMorphs in HandMorphList)
                {
                    if (!toggle_es.val)
                    {
                        morphFile["Left " + HandMorphs] = morphControl.GetMorphByDisplayName("Left " + HandMorphs).morphValue.ToString();
                        morphFile["Right " + HandMorphs] = morphControl.GetMorphByDisplayName("Right " + HandMorphs).morphValue.ToString();
                    }
                    else
                    {
                        if (toggle_er.val)
                        {
                            morphFile["Right " + HandMorphs] = morphControl.GetMorphByDisplayName("Right " + HandMorphs).morphValue.ToString();
                        }
                        if (toggle_el.val)
                        {
                            morphFile["Left " + HandMorphs] = morphControl.GetMorphByDisplayName("Left " + HandMorphs).morphValue.ToString();
                        }
                    }
                }
                SuperController.singleton.DoSaveScreenshot(path);
                SaveJSON(morphFile, path);
            }
            else
            {
                morphFile["setUnlistedParamsToDefault"] = "true";
                morphFile["V2"] = "true";
                morphFile["storables"][0]["id"] = "geometry";

                int i = 0;
                foreach (string HandMorphs in HandMorphList)
                {
                    if (!toggle_es.val)
                    {
                        morphFile["storables"][0]["morphs"][i * 2]["uid"] = morphControl.GetMorphByDisplayName("Left " + HandMorphs).uid;
                        morphFile["storables"][0]["morphs"][i * 2]["name"] = morphControl.GetMorphByDisplayName("Left " + HandMorphs).displayName;
                        morphFile["storables"][0]["morphs"][i * 2]["value"] = morphControl.GetMorphByDisplayName("Left " + HandMorphs).morphValue.ToString();
                        morphFile["storables"][0]["morphs"][i * 2 + 1]["uid"] = morphControl.GetMorphByDisplayName("Right " + HandMorphs).uid;
                        morphFile["storables"][0]["morphs"][i * 2 + 1]["name"] = morphControl.GetMorphByDisplayName("Right " + HandMorphs).displayName;
                        morphFile["storables"][0]["morphs"][i * 2 + 1]["value"] = morphControl.GetMorphByDisplayName("Right " + HandMorphs).morphValue.ToString();
                    }
                    else
                    {
                        if (toggle_er.val)
                        {
                            morphFile["storables"][0]["morphs"][i]["uid"] = morphControl.GetMorphByDisplayName("Right " + HandMorphs).uid;
                            morphFile["storables"][0]["morphs"][i]["name"] = morphControl.GetMorphByDisplayName("Right " + HandMorphs).displayName;
                            morphFile["storables"][0]["morphs"][i]["value"] = morphControl.GetMorphByDisplayName("Right " + HandMorphs).morphValue.ToString();
                        }
                        if (toggle_el.val)
                        {
                            morphFile["storables"][0]["morphs"][i]["uid"] = morphControl.GetMorphByDisplayName("Left " + HandMorphs).uid;
                            morphFile["storables"][0]["morphs"][i]["name"] = morphControl.GetMorphByDisplayName("Left " + HandMorphs).displayName;
                            morphFile["storables"][0]["morphs"][i]["value"] = morphControl.GetMorphByDisplayName("Left " + HandMorphs).morphValue.ToString();
                        }
                    }
                    i++;
                }

                if (path.Substring(path.Length - 4, 4) != ".vap")
                {
                    jsonPath = path + ".json";
                    path = path + ".vap";
                }
                else
                {
                    jsonPath = path.Substring(0, (path.Length - 4)) + ".json";
                }
                SuperController.singleton.DoSaveScreenshot(jsonPath);
                SaveJSON(morphFile, path);
            }
        }
        #endregion

        public void FixedUpdate()
        {
            sliderValueToMorph();
        }
    }
}

